#include "HEPMC3event.h"


#include "HepMC3/WriterAscii.h"

HEPMC3event::HEPMC3event() :
  m_filename("output.hepmc3"),
  evt(Units::GEV,Units::MM)
{

};

HEPMC3event::HEPMC3event(std::string filename) :
  m_filename(filename),
  evt(Units::GEV,Units::MM)
{

};

void HEPMC3event::Initialise()
{
  std::shared_ptr<GenRunInfo> runInfo = std::make_shared<GenRunInfo>();

  // struct GenRunInfo::ToolInfo
  // generator={std::string("GiBUU"),std::to_string(PYTHIA_VERSION).substr(0,5),std::string("Used generator")};

  struct GenRunInfo::ToolInfo generator={
    std::string("GiBUU"),
    std::string(GIBUU_VERSION), // ("v2023"),
    std::string("Used generator")};
  runInfo->tools().push_back(generator);

  struct GenRunInfo::ToolInfo converter={
    std::string("HEPMC3event"),
    std::string("1.0"),
    std::string("Output generator")};
  runInfo->tools().push_back(converter);

  writer = make_shared<WriterAscii>(m_filename,runInfo);
};

void HEPMC3event::InitEvent()
{
  v1 = std::make_shared<GenVertex>();
  v1->set_status(4);
  evt.add_vertex(v1);
};

void HEPMC3event::Write()
{
  writer->write_event(evt);

  // Print::listing(evt);
  // Print::content(evt);

  evt.clear();
};

void HEPMC3event::Close()
{
  writer->close();
};

void HEPMC3event::AddParticleIn(int barcode, double px, double py, double pz, double e, int status)
{
  GenParticlePtr p = std::make_shared<GenParticle>( FourVector(px,py,pz,e),barcode, status );
  v1->add_particle_in(p);
};

void HEPMC3event::AddParticleOut(int barcode, double px, double py, double pz, double e, int status)
{
  GenParticlePtr p = std::make_shared<GenParticle>( FourVector(px,py,pz,e),barcode, status );
  v1->add_particle_out(p);
};

void HEPMC3event::SetWeight(double weight)
{
  std::shared_ptr<Attribute> w = std::make_shared<DoubleAttribute>(weight);
  evt.add_attribute("perWeight", w);

  // std::shared_ptr<GenCrossSection> cross_section = std::make_shared<GenCrossSection>();
  // evt.add_attribute("perWeight",cross_section);

  // cross_section->set_cross_section(weight,0.0);

};

void HEPMC3event::SetPosition(double x, double y, double z)
{
  //  evt.shift_position_to( FourVector(x,y,z,0.0) );
  v1->set_position( FourVector(x,y,z,0.0) );
}

void HEPMC3event::Attribute_impact(double value)
{
  std::shared_ptr<Attribute> w = std::make_shared<DoubleAttribute>(value);
  evt.add_attribute("impact", w);
};

void HEPMC3event::Attribute_eventtype(int value)
{
  std::shared_ptr<Attribute> w = std::make_shared<IntAttribute>(value);
  evt.add_attribute("eventtype", w);
};
